@@ -1,5 +1,5 @@
 /* area.cc -- game widgets for falling blocks game.  -*- C++ -*-
-   Copyright (C) 2011-2012 Gerardo Ballabio
+   Copyright (C) 2011-2014 Gerardo Ballabio
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -142,6 +142,40 @@ void BoardArea::draw_pause_message(Cairo::RefPtr<Cairo::Context> context)
   layout2->get_line(0)->show_in_cairo_context(context);
 }
 
+void BoardArea::draw_game_over_message(Cairo::RefPtr<Cairo::Context> context)
+{
+  // text dimensions
+  Glib::RefPtr<Pango::Layout> layout = create_pango_layout("Game over");
+  Pango::Rectangle extent = layout->get_line(0)->get_pixel_logical_extents();
+  int w = (extent.get_width() - extent.get_lbearing()),
+    h1 = extent.get_height(), h2 = extent.get_descent();
+
+  // center of board
+  Gdk::Rectangle rectangle =
+    g->board_size(brick_size, cells->get_rows(), cells->get_cols(),
+		  cells->get_layers());
+  int cx = (rectangle.get_x() + rectangle.get_width() / 2),
+    cy = (rectangle.get_y() + rectangle.get_height() / 2);
+
+  // draw large font, but make sure that it fits within board
+  double scale = 2.0;
+  if (w * scale > rectangle.get_width())
+    scale = rectangle.get_width() / w;
+  context->scale(scale, scale);
+
+  // draw with extra thickness and inverted color as background
+  context->set_source_rgba(0.0, 0.0, 0.0, 1.0);
+  context->set_line_width(5.0 / scale);
+  context->move_to(cx / scale - w / 2, cy / scale + (h1 - h2) / 2);
+  layout->get_line(0)->add_to_cairo_context(context);
+  context->stroke();
+
+  // draw as foreground
+  context->set_source_rgba(1.0, 1.0, 1.0, 1.0);
+  context->move_to(cx / scale - w / 2, cy / scale + (h1 - h2) / 2);
+  layout->get_line(0)->show_in_cairo_context(context);
+}
+
 bool BoardArea::on_expose_event(GdkEventExpose *event)
 {
   if (!g || !cells)
@@ -186,6 +220,8 @@ bool BoardArea::on_expose_event(GdkEventExpose *event)
 
   if (pause)
     draw_pause_message(context);
+  if (over)
+    draw_game_over_message(context);
 
   return true;
 }
@@ -197,6 +233,7 @@ void BoardArea::start_game(const group *gg, const grid<int> *c,
   cells = c;
   block = b;
   iblock = i % ncolors;
+  pause = over = false;
 
   // set minimum widget size
   Gdk::Rectangle rectangle =
