@@ -50,23 +50,25 @@ private:
   grid<bool> block, shadow;
   int iblock;
   int brick_size; // unit of measure for drawing
-  bool pause;
+  bool pause, over;
   Glib::ustring pause_key;
 
   void draw_2d(Cairo::RefPtr<Cairo::Context> context);
   void draw_3d(Cairo::RefPtr<Cairo::Context> context);
   void draw_pause_message(Cairo::RefPtr<Cairo::Context> context);
+  void draw_game_over_message(Cairo::RefPtr<Cairo::Context> context);
 protected:
   virtual bool on_expose_event(GdkEventExpose *event);
 public:
-  BoardArea() : g(0), cells(0), brick_size(min_brick_size), pause(false)
+  BoardArea() : g(0), cells(0), brick_size(min_brick_size),
+		pause(false), over(false)
   { set_size_request(brick_size * 12, brick_size * 21); }
 
   int get_brick_size() const { return brick_size; }
 
-  // start new game
   void start_game(const group *gg, const grid<int> *c, const grid<bool> &b,
 		  int i);
+  void end_game() { over = true; queue_draw(); }
   void set_block(const grid<bool> &b, const grid<bool> &s, int i);
   void set_pause(bool p) { pause = p; queue_draw(); }
   void set_pause_key(const Glib::ustring &key) { pause_key = key; }
