@@ -1,5 +1,5 @@
 /* grid.h -- three-dimensional grid of points.  -*- C++ -*-
-   Copyright (C) 2011-2013 Gerardo Ballabio
+   Copyright (C) 2011-2014 Gerardo Ballabio
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -123,8 +123,8 @@ template <class T> void grid<T>::add(const coords &c, T v)
       // resize, copy old values
       std::valarray<T> m = mask;
       mask.resize(nrows * ncols * nlayers, T(0));
-      size_t len[] = { rows, cols, layers, };
-      size_t str[] = { ncols, 1, ncols * nrows, };
+      size_t len[] = { size_t(rows), size_t(cols), size_t(layers), };
+      size_t str[] = { size_t(ncols), 1, size_t(ncols * nrows), };
       // doesn't work?
       // std::gslice g(d0.x + ncols * (d0.y + nrows * d0.z),
       // std::valarray<size_t>(len, 3),
