@@ -1,5 +1,5 @@
 /* main.cc -- main window for falling blocks game.  -*- C++ -*-
-   Copyright (C) 2011-2013 Gerardo Ballabio
+   Copyright (C) 2011-2014 Gerardo Ballabio
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -171,15 +171,15 @@ void MainWindow::on_game_new()
   int bmax = preferences.get_bricks_max(),
     bmin = preferences.get_bricks_min();
 
+  // empty cells per line
+  int empty = preferences.get_empty();
+
   // preview next block
   next = preferences.get_next();
 
   // show where block will land
   land = preferences.get_land();
 
-  // empty cells per line
-  int empty = preferences.get_empty();
-
   // start game
   animation = 0;
   pause = false;
@@ -244,8 +244,8 @@ void MainWindow::on_game_scores()
 		    preferences.get_width(), preferences.get_depth(),
 		    preferences.get_mode1(), preferences.get_mode2(),
 		    preferences.get_speed(), preferences.get_bricks_max(),
-		    preferences.get_bricks_min(), preferences.get_next(),
-		    preferences.get_land(), preferences.get_empty());
+		    preferences.get_bricks_min(), preferences.get_empty(),
+		    preferences.get_next(), preferences.get_land());
     }
 
   scores.show();
@@ -253,15 +253,15 @@ void MainWindow::on_game_scores()
 
 void MainWindow::on_help_contents()
 {
-  // use system until I figure out how to do it properly
-  std::system("yelp ghelp:galois &");
+  // launch help browser
+  gtk_show_uri(get_screen()->gobj(), "ghelp:galois", GDK_CURRENT_TIME, 0);
 }
 
 void MainWindow::on_help_about()
 {
   Gtk::AboutDialog dialog;
   dialog.set_name("Galois");
-  dialog.set_version("0.2");
+  dialog.set_version("0.3");
   dialog.set_comments("A falling blocks game with many different geometries");
   dialog.set_license
     ("Galois is free software; you can redistribute it and/or modify\n"
@@ -383,6 +383,7 @@ void MainWindow::on_game_next()
 
 void MainWindow::on_game_over()
 {
+  board_area.end_game();
   next_area.set_block(grid<bool>(), 0);
 
   // stop timeout
@@ -396,15 +397,16 @@ void MainWindow::on_game_over()
   timeval.assign_current_time();
   double time = timeval.as_double();
   scores.add(name, time,
-	     board.get_score(), board.get_lines(), board.get_level(),
-	     board.get_speed(), board.get_bmax(), board.get_bmin(),
+	     board.get_score(), board.get_lines(),
+	     board.get_level(), board.get_speed(),
+	     board.get_bmax(), board.get_bmin(), board.get_bsuper(),
 	     preferences.get_geometry(), preferences.get_reflection(),
 	     preferences.get_width(), preferences.get_depth(),
 	     preferences.get_mode1(), preferences.get_mode2(),
 	     preferences.get_speed(),
 	     preferences.get_bricks_max(), preferences.get_bricks_min(),
-	     preferences.get_next(), preferences.get_land(),
-	     preferences.get_empty());
+	     preferences.get_empty(),
+	     preferences.get_next(), preferences.get_land());
   if (scores.is_active())
     scores.show();
 
@@ -421,7 +423,7 @@ MainWindow::MainWindow()
 {
   add(vbox_all);
 
-  // menu bar and tool bar
+  // menu bar
   actiongroup = Gtk::ActionGroup::create();
   actiongroup->add(Gtk::Action::create("GameMenu", "_Game"));
   actiongroup->add(Gtk::Action::create("GameNew", Gtk::Stock::NEW),
@@ -457,19 +459,17 @@ MainWindow::MainWindow()
     "      <menuitem action='HelpAbout'/>"
     "    </menu>"
     "  </menubar>"
-    "  <toolbar name='ToolBar'>"
-    "    <toolitem action='GameNew'/>"
-    "  </toolbar>"
     "</ui>";
   uimanager->add_ui_from_string(ui_info);
   add_accel_group(uimanager->get_accel_group());
   vbox_all.pack_start(vbox_ui, Gtk::PACK_SHRINK);
   Gtk::Widget *menubar = uimanager->get_widget("/MenuBar");
   vbox_ui.pack_start(*menubar, Gtk::PACK_SHRINK);
-  // Gtk::Widget *toolbar = uimanager->get_widget("/ToolBar");
-  // vbox_ui.pack_start(*toolbar, Gtk::PACK_SHRINK);
+
+  // set initial actions status
   actiongroup->get_action("GameStop")->set_sensitive(false);
 
+  // game elements
   vbox_all.pack_start(hbox, Gtk::PACK_EXPAND_WIDGET);
   hbox.pack_start(board_area, Gtk::PACK_EXPAND_WIDGET);
   hbox.pack_start(vbox, Gtk::PACK_EXPAND_WIDGET);
