@@ -1,5 +1,5 @@
 /* preferences.cc -- preferences dialog for falling blocks game.  -*- C++ -*-
-   Copyright (C) 2011-2013 Gerardo Ballabio
+   Copyright (C) 2011-2014 Gerardo Ballabio
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -207,27 +207,28 @@ PrefsDialog::PrefsDialog()
   depth_adjust.set_value(width3);
   depth_box.set_sensitive(false);
 
-  tab_game.pack_start(level_box1, Gtk::PACK_SHRINK);
-  level_box1.set_spacing(6);
-  level_box1.pack_start(level_heading);
-  level_heading.set_text("<span weight=\"bold\">Levels</span>");
-  level_heading.set_use_markup(true);
-  level_heading.set_alignment(Gtk::ALIGN_LEFT);
-  level_box1.pack_start(level_align);
-  level_align.set_padding(0, 0, 12, 0);
-  level_align.add(level_box2);
-  level_box2.set_spacing(6);
-  level_box2.pack_start(mode_box1);
+  tab_game.pack_start(game_box1, Gtk::PACK_SHRINK);
+  game_box1.set_spacing(6);
+  game_box1.pack_start(game_heading);
+  game_heading.set_text("<span weight=\"bold\">Game</span>");
+  game_heading.set_use_markup(true);
+  game_heading.set_alignment(Gtk::ALIGN_LEFT);
+  game_box1.pack_start(game_align);
+  game_align.set_padding(0, 0, 12, 0);
+  game_align.add(game_box2);
+  game_box2.set_spacing(6);
+  game_box2.pack_start(mode_box1);
   mode_box1.pack_start(mode_label1, Gtk::PACK_SHRINK);
   mode_box1.set_spacing(6);
   mode_label1.set_text("On level change: increase");
   mode_box1.pack_start(mode_combo1, Gtk::PACK_SHRINK);
   mode_combo1.append_text("speed");
   mode_combo1.append_text("max. block size");
+  mode_combo1.append_text("superblock size");
   mode_combo1.set_active(0);
   mode_combo1.signal_changed().connect
     (sigc::mem_fun(*this, &PrefsDialog::on_mode_combo1_signal_changed));
-  level_box2.pack_start(mode_align);
+  game_box2.pack_start(mode_align);
   mode_align.set_padding(0, 0, 12, 0);
   mode_align.add(mode_box2);
   mode_box2.set_spacing(6);
@@ -241,7 +242,7 @@ PrefsDialog::PrefsDialog()
   mode_box2.pack_start(mode_label3, Gtk::PACK_SHRINK);
   mode_label3.set_text("min. block size");
   mode_box2.set_sensitive(false);
-  level_box2.pack_start(speed_box, Gtk::PACK_SHRINK);
+  game_box2.pack_start(speed_box, Gtk::PACK_SHRINK);
   speed_box.set_spacing(6);
   speed_box.pack_start(speed_label, Gtk::PACK_SHRINK);
   speed_label.set_text("Initial speed level:");
@@ -250,7 +251,7 @@ PrefsDialog::PrefsDialog()
   speed_adjust.set_lower(1);
   speed_adjust.set_upper(9);
   speed_adjust.set_value(5);
-  level_box2.pack_start(bricks_box, Gtk::PACK_SHRINK);
+  game_box2.pack_start(bricks_box, Gtk::PACK_SHRINK);
   bricks_box.pack_start(bricks_label1, Gtk::PACK_SHRINK);
   bricks_box.set_spacing(6);
   bricks_label1.set_text("Initial block size: maximum");
@@ -269,22 +270,6 @@ PrefsDialog::PrefsDialog()
   bricks_spin1.signal_value_changed().connect
     (sigc::mem_fun(*this,
 		   &PrefsDialog::on_bricks_spin1_signal_value_changed));
-
-  tab_game.pack_start(game_box1, Gtk::PACK_SHRINK);
-  game_box1.set_spacing(6);
-  game_box1.pack_start(game_heading);
-  game_heading.set_text("<span weight=\"bold\">Game</span>");
-  game_heading.set_use_markup(true);
-  game_heading.set_alignment(Gtk::ALIGN_LEFT);
-  game_box1.pack_start(game_align);
-  game_align.set_padding(0, 0, 12, 0);
-  game_align.add(game_box2);
-  game_box2.set_spacing(6);
-  game_box2.add(next_check);
-  next_check.set_label("Preview next block");
-  next_check.set_active(true);
-  game_box2.add(land_check);
-  land_check.set_label("Show where the block will land");
   game_box2.pack_start(empty_box, Gtk::PACK_SHRINK);
   empty_box.set_spacing(6);
   empty_box.pack_start(empty_label1, Gtk::PACK_SHRINK);
@@ -297,6 +282,22 @@ PrefsDialog::PrefsDialog()
   empty_box.pack_start(empty_label2, Gtk::PACK_SHRINK);
   empty_label2.set_text("empty cells");
 
+  tab_game.pack_start(view_box1, Gtk::PACK_SHRINK);
+  view_box1.set_spacing(6);
+  view_box1.pack_start(view_heading);
+  view_heading.set_text("<span weight=\"bold\">View</span>");
+  view_heading.set_use_markup(true);
+  view_heading.set_alignment(Gtk::ALIGN_LEFT);
+  view_box1.pack_start(view_align);
+  view_align.set_padding(0, 0, 12, 0);
+  view_align.add(view_box2);
+  view_box2.set_spacing(6);
+  view_box2.add(next_check);
+  next_check.set_label("Preview next block");
+  next_check.set_active(true);
+  view_box2.add(land_check);
+  land_check.set_label("Show where the block will land");
+
   // second tab
   notebook.append_page(tab_controls, "Controls");
 
@@ -398,8 +399,11 @@ bool PrefsDialog::load()
   try
     {
       // open file
-      Glib::ustring file = Glib::build_filename(Glib::get_user_config_dir(),
-						"galois.conf");
+      Glib::ustring file =
+	// Debian 6.0 hasn't the three-arguments version
+	// keep supporting it for now
+	Glib::build_filename(Glib::build_filename(Glib::get_user_config_dir(),
+						  "galois"), "galois.conf");
       xmlpp::TextReader *reader = new xmlpp::TextReader(file);
 
       reader->read();
@@ -543,12 +547,12 @@ bool PrefsDialog::load()
 	      else if (name == "bricks_min")
 		bricks_spin2.set_value
 		  (std::atoi(reader->get_value().c_str()));
+	      else if (name == "empty")
+		empty_spin.set_value(std::atoi(reader->get_value().c_str()));
 	      else if (name == "next")
 		next_check.set_active(std::atoi(reader->get_value().c_str()));
 	      else if (name == "land")
 		land_check.set_active(std::atoi(reader->get_value().c_str()));
-	      else if (name == "empty")
-		empty_spin.set_value(std::atoi(reader->get_value().c_str()));
 	    }
 	}
 
@@ -562,9 +566,11 @@ bool PrefsDialog::load()
 
 bool PrefsDialog::save() const
 {
-  // open file
-  Glib::ustring file = Glib::build_filename(Glib::get_user_config_dir(),
-					    "galois.conf");
+  // open file, create directory if doesn't exist
+  Glib::ustring
+    dir = Glib::build_filename(Glib::get_user_config_dir(), "galois"),
+    file = Glib::build_filename(dir, "galois.conf");
+  g_mkdir_with_parents(dir.c_str(), 0755);
   std::ofstream os(file.c_str());
 
   // xml header
@@ -583,9 +589,9 @@ bool PrefsDialog::save() const
   os << "  <speed value=\"" << get_speed() << "\" />" << std::endl;
   os << "  <bricks_max value=\"" << get_bricks_max() << "\" />" << std::endl;
   os << "  <bricks_min value=\"" << get_bricks_min() << "\" />" << std::endl;
+  os << "  <empty value=\"" << get_empty() << "\" />" << std::endl;
   os << "  <next value=\"" << get_next() << "\" />" << std::endl;
   os << "  <land value=\"" << get_land() << "\" />" << std::endl;
-  os << "  <empty value=\"" << get_empty() << "\" />" << std::endl;
   os << "  <keys_2d>" << std::endl;
   for (Gtk::TreeModel::iterator i=two_list->children().begin();
        i!=two_list->children().end(); ++i)
