@@ -1,5 +1,5 @@
 /* scores.cc -- preferences dialog for falling blocks game.  -*- C++ -*-
-   Copyright (C) 2012-2013 Gerardo Ballabio
+   Copyright (C) 2012-2014 Gerardo Ballabio
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,6 +22,40 @@
 #include <libxml++/parsers/textreader.h>
 #include "scores.h"
 
+bool ScoresDialog::pass(const std::vector<game>::const_iterator &i) const
+{
+  // get current filters
+  int geometry = geometry_combo.get_active_row_number(),
+    mode1 = mode1_combo.get_active_row_number();
+  bool reflect_flag = reflect_check.get_active(),
+    mode2_flag = mode2_check.get_active(),
+    size_flag = size_check.get_active(),
+    empty_flag = empty_check.get_active();
+  int reflection = reflect_combo.get_active_row_number(),
+    mode2 = mode2_combo.get_active_row_number(),
+    imax = imax_spin.get_value_as_int(), imin = imin_spin.get_value_as_int(),
+    empty = empty_spin.get_value_as_int();
+
+  // apply filters
+  if (geometry != 4 && i->geometry != geometry)
+    return false;
+  if (mode1 != 3 && i->mode1 != mode1)
+    return false;
+  if (reflect_flag && i->reflection != reflection)
+    return false;
+  if (mode2_flag && mode1 == 1 && i->mode2 != mode2)
+    return false;
+  if (size_flag && i->imax != imax)
+    return false;
+  if (size_flag && i->imin != imin)
+    return false;
+  if (empty_flag && i->empty != empty)
+    return false;
+
+  // passed all filters
+  return true;
+}
+
 void ScoresDialog::make_list(int active)
 {
   list->clear();
@@ -29,32 +63,10 @@ void ScoresDialog::make_list(int active)
   active_row = -1;
   visible_rows = 0;
 
-  // get current filters
-  int geometry = geometry_combo.get_active_row_number(),
-    mode1 = mode1_combo.get_active_row_number();
-  bool reflection = reflect_check.get_active(),
-    mode2 = mode2_check.get_active(), size = size_check.get_active(),
-    empty = empty_check.get_active();
-
   const int max_rows = 10;
   for (std::vector<game>::const_iterator i=games.begin(); i!=games.end(); ++i)
     {
-      // apply filters
-      if (geometry != 4 && i->geometry != geometry)
-	continue;
-      if (mode1 != 2 && i->mode1 != mode1)
-	continue;
-      if (reflection &&
-	  i->reflection != reflect_combo.get_active_row_number())
-	continue;
-      if (mode2 && mode1 == 1 &&
-	  i->mode2 != mode2_combo.get_active_row_number())
-	continue;
-      if (size && i->imax != imax_spin.get_value_as_int())
-	continue;
-      if (size && i->imin != imin_spin.get_value_as_int())
-	continue;
-      if (empty && i->empty != empty_spin.get_value_as_int())
+      if (!pass(i))
 	continue;
 
       Gtk::TreeModel::Row row = *(list->append());
@@ -65,6 +77,7 @@ void ScoresDialog::make_list(int active)
       row[columns.speed] = i->speed;
       row[columns.bmax] = i->bmax;
       row[columns.bmin] = i->bmin;
+      row[columns.bsuper] = i->bsuper;
       if (i - games.begin() == active_game)
 	{
 	  active_row = visible_rows;
@@ -203,6 +216,7 @@ ScoresDialog::ScoresDialog()
   table.attach(mode1_combo, 1, 2, 1, 2);
   mode1_combo.append_text("speed");
   mode1_combo.append_text("block size");
+  mode1_combo.append_text("superblock size");
   mode1_combo.append_text("");
   mode1_combo.set_active(0);
   mode1_combo.signal_changed().connect
@@ -301,6 +315,7 @@ ScoresDialog::ScoresDialog()
   tree.append_column("Speed", columns.speed);
   tree.append_column("Max s.", columns.bmax);
   tree.append_column("Min s.", columns.bmin);
+  tree.append_column("Super s.", columns.bsuper);
   tree.get_column(0)->add_attribute(*tree.get_column_cell_renderer(0),
 				    "editable", columns.editable);
 
@@ -320,37 +335,63 @@ ScoresDialog::ScoresDialog()
 
 void ScoresDialog::add(const Glib::ustring &name, double time,
 		       int score, int lines, int level, int speed,
-		       int bmax, int bmin, int geometry, bool reflection,
-		       int width, int depth, int mode1, int mode2,
-		       int ispeed, int imax, int imin,
-		       bool next, bool land, int empty)
+		       int bmax, int bmin, int bsuper,
+		       int geometry, bool reflection, int width, int depth,
+		       int mode1, int mode2, int ispeed, int imax, int imin,
+		       int empty, bool next, bool land)
 {
-  // find insertion point
-  std::vector<game>::iterator i;
-  for (i=games.begin(); i!=games.end(); ++i)
-    if (i->score < score)
-      break;
-  int n = i - games.begin();
-  games.insert(i, make_game(name, time, score, lines, level,
-			    speed, bmax, bmin, geometry, reflection,
-			    width, depth, mode1, mode2, ispeed, imax, imin,
-			    next, land, empty));
-
-  // write to file
-  save();
-
   // set filters
   filter(geometry, reflection, width, depth, mode1, mode2,
-	 ispeed, imax, imin, next, land, empty);
+	 ispeed, imax, imin, empty, next, land);
+
+  // don't use an iterator as the loop index
+  // because I'm not sure it would still point to the correct place
+  // after an insert or erase
+  unsigned pos = 0, top = 0;
+  int added = -1;
+
+  // skip scores greater than this, count those which pass the filters
+  for (; pos<games.size() && top<10; ++pos)
+    {
+      if (games[pos].score < score)
+	break;
+      if (pass(pos))
+	++top;
+    }
+
+  if (top < 10)
+    {
+      // new score goes here
+      games.insert(games.begin() + pos,
+		   make_game(name, time, score, lines, level, speed,
+			     bmax, bmin, bsuper, geometry, reflection,
+			     width, depth, mode1, mode2, ispeed,
+			     imax, imin, empty, next, land));
+      added = pos;
+      ++pos;
+      ++top;
+    }
+
+  // count scores which pass the filters, drop those outside the top 10
+  for (; pos<games.size(); ++pos)
+    if (pass(pos) && ++top > 10)
+      {
+	games.erase(games.begin() + pos);
+	--pos;
+      }
+
+  // write to file if anything changed
+  if (added >= 0 || top > 10)
+    save();
 
   // show
-  make_list(n);
+  make_list(added);
 }
 
 void ScoresDialog::filter(int geometry, bool reflection,
 			  int width, int depth, int mode1, int mode2,
 			  int ispeed, int imax, int imin,
-			  bool next, bool land, int empty)
+			  int empty, bool next, bool land)
 {
   set_geometry(geometry);
   set_mode1(mode1);
@@ -370,8 +411,11 @@ bool ScoresDialog::load()
   try
     {
       // open file
-      Glib::ustring file = Glib::build_filename(Glib::get_user_data_dir(),
-						"galois.res");
+      Glib::ustring file =
+	// Debian 6.0 hasn't the three-arguments version
+	// keep supporting it for now
+	Glib::build_filename(Glib::build_filename(Glib::get_user_data_dir(),
+						  "galois"), "galois.res");
       xmlpp::TextReader *reader = new xmlpp::TextReader(file);
 
       reader->read();
@@ -429,6 +473,8 @@ bool ScoresDialog::load()
 		    g.bmax = std::atoi(reader->get_value().c_str());
 		  else if (reader->get_name() == "bmin")
 		    g.bmin = std::atoi(reader->get_value().c_str());
+		  else if (reader->get_name() == "bsuper")
+		    g.bsuper = std::atoi(reader->get_value().c_str());
 		  else if (reader->get_name() == "geometry")
 		    g.geometry = std::atoi(reader->get_value().c_str());
 		  else if (reader->get_name() == "reflection")
@@ -447,12 +493,12 @@ bool ScoresDialog::load()
 		    g.imax = std::atoi(reader->get_value().c_str());
 		  else if (reader->get_name() == "imin")
 		    g.imin = std::atoi(reader->get_value().c_str());
+		  else if (reader->get_name() == "empty")
+		    g.empty = std::atoi(reader->get_value().c_str());
 		  else if (reader->get_name() == "next")
 		    g.next = std::atoi(reader->get_value().c_str());
 		  else if (reader->get_name() == "land")
 		    g.land = std::atoi(reader->get_value().c_str());
-		  else if (reader->get_name() == "empty")
-		    g.empty = std::atoi(reader->get_value().c_str());
 		}
 	      while (reader->move_to_next_attribute());
 	    }
@@ -471,9 +517,11 @@ bool ScoresDialog::load()
 
 bool ScoresDialog::save() const
 {
-  // open file
-  Glib::ustring file = Glib::build_filename(Glib::get_user_data_dir(),
-					    "galois.res");
+  // open file, create directory if doesn't exist
+  Glib::ustring
+    dir = Glib::build_filename(Glib::get_user_data_dir(), "galois"),
+    file = Glib::build_filename(dir, "galois.res");
+  g_mkdir_with_parents(dir.c_str(), 0755);
   std::ofstream os(file.c_str());
 
   // xml header
@@ -494,6 +542,7 @@ bool ScoresDialog::save() const
        << " speed=\"" << i->speed << "\""
        << " bmax=\"" << i->bmax << "\""
        << " bmin=\"" << i->bmin << "\""
+       << " bsuper=\"" << i->bsuper << "\""
        << " geometry=\"" << i->geometry << "\""
        << " reflection=\"" << i->reflection << "\""
        << " width=\"" << i->width << "\""
@@ -503,9 +552,9 @@ bool ScoresDialog::save() const
        << " ispeed=\"" << i->ispeed << "\""
        << " imax=\"" << i->imax << "\""
        << " imin=\"" << i->imin << "\""
+       << " empty=\"" << i->empty << "\""
        << " next=\"" << i->next << "\""
        << " land=\"" << i->land << "\""
-       << " empty=\"" << i->empty << "\""
        << " />" << std::endl;
 
   // close document node
